@@ -22,18 +22,20 @@
 
 #include "check_hooks.h"
 #include "color.h"
+#include "xalloc.h"
 
 int found_issue = 0;
 int suppress_output = 0;
+int full_path = 0;
 
 #define ALLOC_NODE(nl)  if (ck->check_nodes[nl]) { \
 		loc = ck->check_nodes[nl]; \
 		while (loc->next) { loc = loc->next; } \
-		loc->next = malloc(sizeof(struct check_node)); \
+		loc->next = xmalloc(sizeof(struct check_node)); \
 		if (!loc->next) { return SELINT_OUT_OF_MEM; } \
 		loc = loc->next; \
 } else { \
-		ck->check_nodes[nl] = malloc(sizeof(struct check_node)); \
+		ck->check_nodes[nl] = xmalloc(sizeof(struct check_node)); \
 		if (!ck->check_nodes[nl]) { return SELINT_OUT_OF_MEM; } \
 		loc = ck->check_nodes[nl]; \
 }
@@ -48,7 +50,7 @@ enum selint_error add_check(enum node_flavor check_flavor, struct checks *ck,
 	ALLOC_NODE(check_flavor);
 
 	loc->check_function = check_function;
-	loc->check_id = strdup(check_id);
+	loc->check_id = xstrdup(check_id);
 	loc->issues_found = 0;
 	loc->next = NULL;
 
@@ -92,18 +94,25 @@ enum selint_error call_checks_for_node_type(struct check_node *ck_list,
 void display_check_result(const struct check_result *res, const struct check_data *data)
 {
 	static const size_t FILENAME_PADDING = 22;
-
-	const size_t len = strlen(data->filename);
+	const char *name;
 	unsigned int padding;
 
-	if (FILENAME_PADDING < len) {
+	if (full_path) {
+		name = data->filepath;
 		padding = 0;
 	} else {
-		padding = (unsigned)(FILENAME_PADDING - len);
+		name = data->filename;
+		const size_t len = strlen(name);
+
+		if (FILENAME_PADDING < len) {
+			padding = 0;
+		} else {
+			padding = (unsigned)(FILENAME_PADDING - len);
+		}
 	}
 
 	printf("%s:%*u: %s(%c)%s: %s (%c-%03u)\n",
-	       data->filename,
+	       name,
 	       padding,
 	       res->lineno,
 	       color_severity(res->severity),
@@ -146,6 +155,9 @@ int is_valid_check(const char *check_str)
 	char severity = check_str[0];
 
 	switch (severity) {
+	case 'X':
+		max_id = X_END - 1;
+		break;
 	case 'C':
 		max_id = C_END - 1;
 		break;
@@ -189,11 +201,22 @@ static unsigned int count_check_nodes(const struct checks *ck)
 	return count;
 }
 
-#define COMPARE_IDS(node1_id, node2_id)\
-if (node1_id == node2_id) {\
-	return 0;\
-} else {\
-	return (node1_id > node2_id?1:-1);\
+static int id_priority(char id)
+{
+	switch (id) {
+	case 'X':
+		return 0;
+	case 'C':
+		return 1;
+	case 'S':
+		return 2;
+	case 'W':
+		return 3;
+	case 'E':
+		return 4;
+	default:
+		return 5; //Should never happen, but no way to return an error
+	}
 }
 
 // Return negative if n1 goes before n2, positive if n1 goes after n2 or equal if they are equivalent
@@ -202,41 +225,18 @@ static int comp_check_nodes(const void *n1, const void *n2)
 	const struct check_node *node1 = *(const struct check_node *const *)n1;
 	const struct check_node *node2 = *(const struct check_node *const *)n2;
 
+	int check1_priority = id_priority(node1->check_id[0]);
+	int check2_priority = id_priority(node2->check_id[0]);
+
+	int r = (check1_priority > check2_priority) - (check1_priority < check2_priority);
+	if (r != 0) {
+		return r;
+	}
+
 	int node1_id = atoi(node1->check_id + 2);
 	int node2_id = atoi(node2->check_id + 2);
 
-	switch (node1->check_id[0]) {
-	case 'C':
-		if (node2->check_id[0] == 'C') {
-			COMPARE_IDS(node1_id, node2_id);
-		} else {
-			return -1;
-		}
-	case 'S':
-		if (node2->check_id[0] == 'C') {
-			return 1;
-		} else if (node2->check_id[0] == 'S') {
-			COMPARE_IDS(node1_id, node2_id);
-		} else {
-			return -1;
-		}
-	case 'W':
-		if (node2->check_id[0] == 'C' || node2->check_id[0] == 'S') {
-			return 1;
-		} else if (node2->check_id[0] == 'W') {
-			COMPARE_IDS(node1_id, node2_id);
-		} else {
-			return -1;
-		}
-	case 'E':
-		if (node2->check_id[0] == 'E') {
-			COMPARE_IDS(node1_id, node2_id);
-		} else {
-			return 1;
-		}
-	default:
-		return 0; //Should never happen, but no way to return an error
-	}
+	return (node1_id > node2_id) - (node1_id < node2_id);
 }
 
 void display_check_issue_counts(const struct checks *ck)
@@ -245,7 +245,7 @@ void display_check_issue_counts(const struct checks *ck)
 	unsigned int printed_something = 0;
 
 	// Build flat array of check nodes
-	struct check_node **node_arr = calloc(num_nodes, sizeof(struct check_node *));
+	struct check_node **node_arr = xcalloc(num_nodes, sizeof(struct check_node *));
 	unsigned int node_arr_index = 0;
 	for (int i=0; i <= NODE_ERROR; i++) {
 		if (ck->check_nodes[i]) {
@@ -304,7 +304,7 @@ struct check_result *make_check_result(char severity, unsigned int check_id,
                                        const char *format, ...)
 {
 
-	struct check_result *res = malloc(sizeof(struct check_result));
+	struct check_result *res = xmalloc(sizeof(struct check_result));
 
 	res->severity = severity;
 	res->check_id = check_id;
@@ -315,7 +315,7 @@ struct check_result *make_check_result(char severity, unsigned int check_id,
 	if (vasprintf(&res->message, format, args) == -1) {
 		res->severity = 'F';
 		res->check_id = F_ID_INTERNAL;
-		res->message = strdup("Failed to generate check result message");
+		res->message = xstrdup("Failed to generate check result message");
 	}
 
 	va_end(args);
