@@ -15,6 +15,7 @@
 */
 
 #include "maps.h"
+#include "xalloc.h"
 
 #if defined(__clang__) && defined(__clang_major__) && (__clang_major__ >= 4)
 #if (__clang_major__ >= 12)
@@ -36,10 +37,7 @@ static struct hash_elem *class_map = NULL;
 static struct hash_elem *perm_map = NULL;
 static struct hash_elem *mods_map = NULL;
 static struct hash_elem *mod_layers_map = NULL;
-static struct hash_elem *ifs_map = NULL;
-static struct bool_hash_elem *transform_map = NULL;
-static struct bool_hash_elem *filetrans_map = NULL;
-static struct bool_hash_elem *role_if_map = NULL;
+static struct if_hash_elem *interfaces_map = NULL;
 static struct sl_hash_elem *permmacros_map = NULL;
 static struct template_hash_elem *template_map = NULL;
 
@@ -94,9 +92,9 @@ void insert_into_decl_map(const char *name, const char *module_name,
 
 	if (decl == NULL) {     // Item not in hash table already
 
-		decl = malloc(sizeof(struct hash_elem));
-		decl->key = strdup(name);
-		decl->val = strdup(module_name);
+		decl = xmalloc(sizeof(struct hash_elem));
+		decl->key = xstrdup(name);
+		decl->val = xstrdup(module_name);
 
 		switch (flavor) {
 		case DECL_TYPE:
@@ -161,9 +159,9 @@ void insert_into_mods_map(const char *mod_name, const char *status)
 	HASH_FIND(hh_mods, mods_map, mod_name, strlen(mod_name), mod);
 
 	if (!mod) {
-		mod = malloc(sizeof(struct hash_elem));
-		mod->key = strdup(mod_name);
-		mod->val = strdup(status);
+		mod = xmalloc(sizeof(struct hash_elem));
+		mod->key = xstrdup(mod_name);
+		mod->val = xstrdup(status);
 		HASH_ADD_KEYPTR(hh_mods, mods_map, mod->key, strlen(mod->key),
 		                mod);
 	}
@@ -192,9 +190,9 @@ void insert_into_mod_layers_map(const char *mod_name, const char *layer)
 	HASH_FIND(hh_mod_layers, mod_layers_map, mod_name, strlen(mod_name), mod);
 
 	if (!mod) {
-		mod = malloc(sizeof(struct hash_elem));
-		mod->key = strdup(mod_name);
-		mod->val = strdup(layer);
+		mod = xmalloc(sizeof(struct hash_elem));
+		mod->key = xstrdup(mod_name);
+		mod->val = xstrdup(layer);
 		HASH_ADD_KEYPTR(hh_mod_layers, mod_layers_map, mod->key, strlen(mod->key),
 		                mod);
 	}
@@ -219,16 +217,20 @@ no_sanitize_unsigned_integer_
 void insert_into_ifs_map(const char *if_name, const char *mod_name)
 {
 
-	struct hash_elem *if_call;
+	struct if_hash_elem *if_call;
 
-	HASH_FIND(hh_ifs, ifs_map, if_name, strlen(if_name), if_call);
+	HASH_FIND(hh_interfaces, interfaces_map, if_name, strlen(if_name), if_call);
 
 	if (!if_call) {
-		if_call = malloc(sizeof(struct hash_elem));
-		if_call->key = strdup(if_name);
-		if_call->val = strdup(mod_name);
-		HASH_ADD_KEYPTR(hh_ifs, ifs_map, if_call->key,
-		                strlen(if_call->key), if_call);
+		if_call = xmalloc(sizeof(struct if_hash_elem));
+		if_call->name = xstrdup(if_name);
+		if_call->module = xstrdup(mod_name);
+		if_call->flags = 0;
+		HASH_ADD_KEYPTR(hh_interfaces, interfaces_map, if_call->name,
+				strlen(if_call->name), if_call);
+	} else {
+		free(if_call->module);
+		if_call->module = xstrdup(mod_name);
 	}
 }
 
@@ -236,14 +238,14 @@ no_sanitize_unsigned_integer_
 const char *look_up_in_ifs_map(const char *if_name)
 {
 
-	struct hash_elem *if_call;
+	struct if_hash_elem *if_call;
 
-	HASH_FIND(hh_ifs, ifs_map, if_name, strlen(if_name), if_call);
+	HASH_FIND(hh_interfaces, interfaces_map, if_name, strlen(if_name), if_call);
 
 	if (if_call == NULL) {
 		return NULL;
 	} else {
-		return if_call->val;
+		return if_call->module;
 	}
 }
 
@@ -274,27 +276,28 @@ unsigned int decl_map_count(enum decl_flavor flavor)
 no_sanitize_unsigned_integer_
 void mark_transform_if(const char *if_name)
 {
-	struct bool_hash_elem *transform_if;
+	struct if_hash_elem *transform_if;
 
-	HASH_FIND(hh_transform, transform_map, if_name, strlen(if_name), transform_if);
+	HASH_FIND(hh_interfaces, interfaces_map, if_name, strlen(if_name), transform_if);
 
 	if (!transform_if) {
-		transform_if = malloc(sizeof(struct bool_hash_elem));
-		transform_if->key = strdup(if_name);
-		transform_if->val = 1;
-		HASH_ADD_KEYPTR(hh_transform, transform_map, transform_if->key,
-		                strlen(transform_if->key), transform_if);
+		transform_if = xmalloc(sizeof(struct if_hash_elem));
+		transform_if->name = xstrdup(if_name);
+		transform_if->module = NULL;
+		transform_if->flags = TRANSFORM_IF;
+		HASH_ADD_KEYPTR(hh_interfaces, interfaces_map, transform_if->name,
+				strlen(transform_if->name), transform_if);
 	} else {
-		transform_if->val = 1;
+		transform_if->flags |= TRANSFORM_IF;
 	}
 }
 
 no_sanitize_unsigned_integer_
 int is_transform_if(const char *if_name)
 {
-	struct bool_hash_elem *transform_if;
-	HASH_FIND(hh_transform, transform_map, if_name, strlen(if_name), transform_if);
-	if (transform_if && transform_if->val == 1) {
+	struct if_hash_elem *transform_if;
+	HASH_FIND(hh_interfaces, interfaces_map, if_name, strlen(if_name), transform_if);
+	if (transform_if && (transform_if->flags & TRANSFORM_IF)) {
 		return 1;
 	} else {
 		return 0;
@@ -304,27 +307,28 @@ int is_transform_if(const char *if_name)
 no_sanitize_unsigned_integer_
 void mark_filetrans_if(const char *if_name)
 {
-	struct bool_hash_elem *filetrans_if;
+	struct if_hash_elem *filetrans_if;
 
-	HASH_FIND(hh_filetrans, filetrans_map, if_name, strlen(if_name), filetrans_if);
+	HASH_FIND(hh_interfaces, interfaces_map, if_name, strlen(if_name), filetrans_if);
 
 	if (!filetrans_if) {
-		filetrans_if = malloc(sizeof(struct bool_hash_elem));
-		filetrans_if->key = strdup(if_name);
-		filetrans_if->val = 1;
-		HASH_ADD_KEYPTR(hh_filetrans, filetrans_map, filetrans_if->key,
-		                strlen(filetrans_if->key), filetrans_if);
+		filetrans_if = xmalloc(sizeof(struct if_hash_elem));
+		filetrans_if->name = xstrdup(if_name);
+		filetrans_if->module = NULL;
+		filetrans_if->flags = FILETRANS_IF;
+		HASH_ADD_KEYPTR(hh_interfaces, interfaces_map, filetrans_if->name,
+				strlen(filetrans_if->name), filetrans_if);
 	} else {
-		filetrans_if->val = 1;
+		filetrans_if->flags |= FILETRANS_IF;
 	}
 }
 
 no_sanitize_unsigned_integer_
 int is_filetrans_if(const char *if_name)
 {
-	struct bool_hash_elem *filetrans_if;
-	HASH_FIND(hh_filetrans, filetrans_map, if_name, strlen(if_name), filetrans_if);
-	if (filetrans_if && filetrans_if->val == 1) {
+	struct if_hash_elem *filetrans_if;
+	HASH_FIND(hh_interfaces, interfaces_map, if_name, strlen(if_name), filetrans_if);
+	if (filetrans_if && (filetrans_if->flags & FILETRANS_IF)) {
 		return 1;
 	} else {
 		return 0;
@@ -334,27 +338,69 @@ int is_filetrans_if(const char *if_name)
 no_sanitize_unsigned_integer_
 void mark_role_if(const char *if_name)
 {
-	struct bool_hash_elem *role_if;
+	struct if_hash_elem *role_if;
 
-	HASH_FIND(hh_role_if, role_if_map, if_name, strlen(if_name), role_if);
+	HASH_FIND(hh_interfaces, interfaces_map, if_name, strlen(if_name), role_if);
 
 	if (!role_if) {
-		role_if = malloc(sizeof(struct bool_hash_elem));
-		role_if->key = strdup(if_name);
-		role_if->val = 1;
-		HASH_ADD_KEYPTR(hh_role_if, role_if_map, role_if->key,
-		                strlen(role_if->key), role_if);
+		role_if = xmalloc(sizeof(struct if_hash_elem));
+		role_if->name = xstrdup(if_name);
+		role_if->module = NULL;
+		role_if->flags = ROLE_IF;
+		HASH_ADD_KEYPTR(hh_interfaces, interfaces_map, role_if->name,
+				strlen(role_if->name), role_if);
 	} else {
-		role_if->val = 1;
+		role_if->flags |= ROLE_IF;
 	}
 }
 
 no_sanitize_unsigned_integer_
 int is_role_if(const char *if_name)
 {
-	struct bool_hash_elem *role_if;
-	HASH_FIND(hh_role_if, role_if_map, if_name, strlen(if_name), role_if);
-	if (role_if && role_if->val == 1) {
+	struct if_hash_elem *role_if;
+	HASH_FIND(hh_interfaces, interfaces_map, if_name, strlen(if_name), role_if);
+	if (role_if && (role_if->flags & ROLE_IF)) {
+		return 1;
+	} else {
+		return 0;
+	}
+}
+
+#if defined(__clang__) && defined(__clang_major__) && (__clang_major__ >= 4)
+__attribute__((no_sanitize("unsigned-integer-overflow")))
+#if (__clang_major__ >= 12)
+__attribute__((no_sanitize("unsigned-shift-base")))
+#endif
+#endif
+void mark_used_if(const char *if_name)
+{
+	struct if_hash_elem *used_if;
+
+	HASH_FIND(hh_interfaces, interfaces_map, if_name, strlen(if_name), used_if);
+
+	if (!used_if) {
+		used_if = xmalloc(sizeof(struct if_hash_elem));
+		used_if->name = xstrdup(if_name);
+		used_if->module = NULL;
+		used_if->flags = USED_IF;
+		HASH_ADD_KEYPTR(hh_interfaces, interfaces_map, used_if->name,
+				strlen(used_if->name), used_if);
+	} else {
+		used_if->flags |= USED_IF;
+	}
+}
+
+#if defined(__clang__) && defined(__clang_major__) && (__clang_major__ >= 4)
+__attribute__((no_sanitize("unsigned-integer-overflow")))
+#if (__clang_major__ >= 12)
+__attribute__((no_sanitize("unsigned-shift-base")))
+#endif
+#endif
+int is_used_if(const char *if_name)
+{
+	struct if_hash_elem *used_if;
+	HASH_FIND(hh_interfaces, interfaces_map, if_name, strlen(if_name), used_if);
+	if (used_if && (used_if->flags & USED_IF)) {
 		return 1;
 	} else {
 		return 0;
@@ -404,8 +450,8 @@ static void insert_into_template_map(const char *name, void *new_node,
 	HASH_FIND(hh, template_map, name, strlen(name), template);
 
 	if (template == NULL) {
-		template = malloc(sizeof(struct template_hash_elem));
-		template->name = strdup(name);
+		template = xmalloc(sizeof(struct template_hash_elem));
+		template->name = xstrdup(name);
 		template->declarations = NULL;
 		template->calls = NULL;
 
@@ -427,13 +473,13 @@ void insert_decl_into_template_map(const char *name, enum decl_flavor flavor,
 {
 
 	struct declaration_data *new_data =
-		malloc(sizeof(struct declaration_data));
+		xmalloc(sizeof(struct declaration_data));
 
 	new_data->flavor = flavor;
-	new_data->name = strdup(declaration);
+	new_data->name = xstrdup(declaration);
 	new_data->attrs = NULL; //Not needed
 
-	struct decl_list *new_node = malloc(sizeof(struct decl_list));
+	struct decl_list *new_node = xmalloc(sizeof(struct decl_list));
 	new_node->decl = new_data;
 	new_node->next = NULL;
 
@@ -443,7 +489,7 @@ void insert_decl_into_template_map(const char *name, enum decl_flavor flavor,
 void insert_call_into_template_map(const char *name, struct if_call_data *call)
 {
 
-	struct if_call_list *new_node = malloc(sizeof(struct if_call_list));
+	struct if_call_list *new_node = xmalloc(sizeof(struct if_call_list));
 
 	new_node->call = call;
 	new_node->next = NULL;
@@ -493,8 +539,8 @@ void insert_into_permmacros_map(const char *name, struct string_list *permission
 	HASH_FIND(hh_permmacros, permmacros_map, name, strlen(name), perm_macro);
 
 	if (!perm_macro) {
-		perm_macro = malloc(sizeof(struct sl_hash_elem));
-		perm_macro->key = strdup(name);
+		perm_macro = xmalloc(sizeof(struct sl_hash_elem));
+		perm_macro->key = xstrdup(name);
 		perm_macro->val = permissions;
 		HASH_ADD_KEYPTR(hh_permmacros, permmacros_map, perm_macro->key, strlen(perm_macro->key),
 		                perm_macro);
@@ -525,7 +571,7 @@ void visit_all_in_permmacros_map(void (*visitor)(const char *name, const struct
 	}
 }
 
-unsigned int permmacros_map_count()
+unsigned int permmacros_map_count(void)
 {
 	return HASH_CNT(hh_permmacros, permmacros_map);
 }
@@ -537,13 +583,14 @@ unsigned int permmacros_map_count()
 		free(cur_decl); \
 } \
 
-#define FREE_BOOL_MAP(mn) HASH_ITER(hh_ ## mn, mn ## _map, cur_bool, tmp_bool) { \
-		HASH_DELETE(hh_ ## mn, mn ## _map, cur_bool); \
-		free(cur_bool->key); \
-		free(cur_bool); \
+#define FREE_IF_MAP(mn) HASH_ITER(hh_ ## mn, mn ## _map, cur_if, tmp_if) { \
+		HASH_DELETE(hh_ ## mn, mn ## _map, cur_if); \
+		free(cur_if->name); \
+		free(cur_if->module); \
+		free(cur_if); \
 } \
 
-void free_all_maps()
+void free_all_maps(void)
 {
 
 	struct hash_elem *cur_decl, *tmp_decl;
@@ -568,15 +615,9 @@ void free_all_maps()
 
 	FREE_MAP(mod_layers);
 
-	FREE_MAP(ifs);
+	struct if_hash_elem *cur_if, *tmp_if;
 
-	struct bool_hash_elem *cur_bool, *tmp_bool;
-
-	FREE_BOOL_MAP(transform);
-
-	FREE_BOOL_MAP(filetrans);
-
-	FREE_BOOL_MAP(role_if);
+	FREE_IF_MAP(interfaces);
 
 	struct sl_hash_elem *cur_sl, *tmp_sl;
 
