@@ -22,6 +22,7 @@
 
 #include "ordering.h"
 #include "maps.h"
+#include "xalloc.h"
 
 #define SECTION_NON_ORDERED "_non_ordered"
 #define SECTION_DECLARATION "_declaration"
@@ -46,7 +47,7 @@ struct ordering_metadata *prepare_ordering_metadata(const struct check_data *dat
 {
 	const struct policy_node *cur = head->next; // head is file.  Order the contents
 	size_t count = 0;
-	struct section_data *sections = calloc(1, sizeof(struct section_data));
+	struct section_data *sections = xcalloc(1, sizeof(struct section_data));
 
 	while (cur) {
 		if (add_section_info(sections, get_section(cur), cur->lineno) == SELINT_BAD_ARG) {
@@ -58,7 +59,7 @@ struct ordering_metadata *prepare_ordering_metadata(const struct check_data *dat
 	}
 	calculate_average_lines(sections);
 
-	struct ordering_metadata *ret = calloc(1, sizeof(struct ordering_metadata) +
+	struct ordering_metadata *ret = xcalloc(1, sizeof(struct ordering_metadata) +
 	                                       (count * sizeof(struct order_node)));
 	ret->mod_name = data->mod_name; // Will only be needed for duration of check, so will remain allocated
 	                                // until we are done with this copy
@@ -124,12 +125,29 @@ void calculate_longest_increasing_subsequence(const struct policy_node *head,
 	}
 
 #ifdef DEBUG_INFO
-	for (int i=0; i< ordering->order_node_len; i++) {
+	enum order_conf variant;
+	if (comp_func == compare_nodes_refpolicy) {
+		variant = ORDER_REF;
+	} else if (comp_func == compare_nodes_refpolicy_lax) {
+		variant = ORDER_LAX;
+	} else if (comp_func == compare_nodes_refpolicy_light) {
+		variant = ORDER_LIGHT;
+	} else {
+		variant = ORDER_REF; // Should never happen
+	}
+
+	for (size_t i=0; i< ordering->order_node_len; i++) {
 		if(nodes[i].node) {
-			printf("Line: %u, Section %s: LSS: %d\n",
+			const char *section = get_section(nodes[i].node);
+			printf("Line: %u, Section %s: LSS: %s in-order=%u index=%zu seq_prev=%d end_of_seq=%d avg_line=%.1f\n",
 			       nodes[i].node->lineno,
-			       get_section(nodes[i].node),
-			       get_local_subsection(ordering->mod_name, nodes[i].node));
+			       section,
+			       lss_to_string(get_local_subsection(ordering->mod_name, nodes[i].node, variant)),
+			       nodes[i].in_order,
+			       i,
+			       nodes[i].seq_prev,
+			       nodes[i].end_of_seq,
+			       get_avg_line_by_name(section, ordering->sections));
 		}
 	}
 #endif
@@ -146,7 +164,7 @@ enum selint_error add_section_info(struct section_data *sections,
 	if (sections->section_name != NULL) {
 		while (0 != strcmp(cur->section_name, section_name)) {
 			if (cur->next == NULL) {
-				cur->next = calloc(1, sizeof(struct section_data));
+				cur->next = xcalloc(1, sizeof(struct section_data));
 				cur = cur->next;
 				break;
 			}
@@ -156,7 +174,7 @@ enum selint_error add_section_info(struct section_data *sections,
 	// cur is now the appropriate section_data node.  If section_name is
 	// NULL, then this is a new node
 	if (!cur->section_name) {
-		cur->section_name = strdup(section_name);
+		cur->section_name = xstrdup(section_name);
 	}
 
 	cur->lineno_count++;
@@ -176,6 +194,7 @@ const char *get_section(const struct policy_node *node)
 	case NODE_FC_FILE:
 	case NODE_SPT_FILE:
 	case NODE_AV_FILE:
+	case NODE_COND_FILE:
 		return NULL; // Should never happen
 	case NODE_HEADER:
 		return SECTION_NON_ORDERED; // Guaranteed at top by grammar
@@ -307,10 +326,22 @@ float get_avg_line_by_name(const char *section_name, const struct section_data *
 
 static bool is_self_rule(const struct policy_node *node)
 {
-	return (node->flavor == NODE_AV_RULE || node->flavor == NODE_XAV_RULE) &&
+	// Access vector rule with self as target
+	if ((node->flavor == NODE_AV_RULE || node->flavor == NODE_XAV_RULE) &&
 	       node->data.av_data &&
 	       node->data.av_data->targets &&
-	       0 == strcmp(node->data.av_data->targets->string, "self");
+	       0 == strcmp(node->data.av_data->targets->string, "self")) {
+		return true;
+	}
+
+	// Macro call with self as argument
+	if (node->flavor == NODE_IF_CALL &&
+	    str_in_sl("self", node->data.ic_data->args) &&
+	    !look_up_in_ifs_map(node->data.ic_data->name)) {
+		return true;
+	}
+
+	return false;
 }
 
 static bool is_own_module_rule(const struct policy_node *node, const char *current_mod_name)
@@ -462,6 +493,19 @@ static bool is_optional(const struct policy_node *node)
 	return ret;
 }
 
+static unsigned optional_depth(const struct policy_node *node)
+{
+	unsigned ret = 0;
+	while (node) {
+		if (node->flavor == NODE_OPTIONAL_POLICY ||
+		    node->flavor == NODE_OPTIONAL_ELSE) {
+			ret++;
+		}
+		node = node->parent;
+	}
+	return ret;
+}
+
 static bool is_boolean(const struct policy_node *node)
 {
 	bool ret = false;
@@ -613,9 +657,7 @@ enum order_difference_reason compare_nodes_refpolicy_generic(const struct orderi
 		if (0 != strcmp(first_section_name, SECTION_DECLARATION) &&
 		    (0 == strcmp(second_section_name, SECTION_DECLARATION) ||
 		     (get_avg_line_by_name(first_section_name, ordering_data->sections) >
-		      get_avg_line_by_name(second_section_name, ordering_data->sections))) &&
-		    // allow raw section alphabetically following another raw section
-		    (!(first_section_name[0] != '_' && second_section_name[0] != '_' && strcmp(first_section_name, second_section_name) < 0))) {
+		      get_avg_line_by_name(second_section_name, ordering_data->sections)))) {
 			return -ORDER_SECTION;
 		} else {
 			return ORDER_SECTION;
@@ -675,8 +717,11 @@ enum order_difference_reason compare_nodes_refpolicy_generic(const struct orderi
 		    lss_first != LSS_BOOLEAN &&
 		    lss_first != LSS_TUNABLE &&
 		    (lss_first != LSS_OPTIONAL ||
-		    // sort optionals only based on first node
-		    (lss_first == lss_second && first->prev->flavor == NODE_START_BLOCK && second->prev->flavor == NODE_START_BLOCK)) &&
+		    // sort optionals only based on first node with same depth
+		     (lss_first == lss_second &&
+		     first->prev->flavor == NODE_START_BLOCK &&
+		     second->prev->flavor == NODE_START_BLOCK &&
+		     optional_depth(first) == optional_depth(second))) &&
 		    first->flavor == NODE_IF_CALL &&
 		    second->flavor == NODE_IF_CALL) {
 			const char *first_mod_name = look_up_in_ifs_map(first->data.ic_data->name);
@@ -760,7 +805,7 @@ const char *lss_to_string(enum local_subsection lss)
 	}
 }
 
-char *get_ordering_reason(struct ordering_metadata *order_data, unsigned int index, enum order_conf variant)
+char *get_ordering_reason(const struct ordering_metadata *order_data, unsigned int index, enum order_conf variant)
 {
 	unsigned int distance = 1;
 	unsigned int nearest_index = 0;
@@ -888,13 +933,13 @@ char *get_ordering_reason(struct ordering_metadata *order_data, unsigned int ind
 		if (other_lss == LSS_KERNEL || other_lss == LSS_SYSTEM || other_lss == LSS_OTHER) {
 			enum local_subsection this_lss = get_local_subsection(order_data->mod_name, this_node, variant);
 			if (this_lss == LSS_KERNEL) {
-				followup_str = strdup("  (This interface is in the kernel layer.)");
+				followup_str = xstrdup("  (This interface is in the kernel layer.)");
 			} else if (this_lss == LSS_SYSTEM) {
-				followup_str = strdup("  (This interface is in the system layer.)");
+				followup_str = xstrdup("  (This interface is in the system layer.)");
 			} else if (this_lss == LSS_OTHER) {
-				followup_str = strdup("  (This interface is in a layer other than kernel or system.)");
+				followup_str = xstrdup("  (This interface is in a layer other than kernel or system.)");
 			} else if (this_lss == LSS_KERNEL_MOD) {
-				followup_str = strdup("  (This interface is in the kernel module.)");
+				followup_str = xstrdup("  (This interface is in the kernel module.)");
 			}
 			// Otherwise, it's not an interface call and is hopefully obvious to the user what layer its in
 		}
@@ -925,7 +970,7 @@ char *get_ordering_reason(struct ordering_metadata *order_data, unsigned int ind
 		str_len += strlen(followup_str);
 	}
 
-	char *ret = malloc(sizeof(char) * str_len);
+	char *ret = xmalloc(sizeof(char) * str_len);
 
 	ssize_t written = snprintf(ret, str_len,
 	                           "Line out of order.  It is of type %s %s line %u %s.",
