@@ -25,7 +25,7 @@
 #include "perm_macro.h"
 #include "xalloc.h"
 
-char *module_name = NULL;
+static char *module_name = NULL;
 
 enum selint_error insert_header(struct policy_node **cur, const char *mn,
                                 enum header_flavor flavor, unsigned int lineno)
@@ -613,7 +613,7 @@ enum selint_error begin_boolean_policy(struct policy_node **cur,
                                        unsigned int lineno)
 {
 	struct cond_declaration_data *cd_data = xmalloc(sizeof(struct cond_declaration_data));
-	cd_data->identifiers = NULL;;
+	cd_data->identifiers = NULL;
 
 	union node_data nd;
 	nd.cd_data = cd_data;
@@ -631,7 +631,7 @@ enum selint_error begin_tunable_policy(struct policy_node **cur,
                                        unsigned int lineno)
 {
 	struct cond_declaration_data *cd_data = xmalloc(sizeof(struct cond_declaration_data));
-	cd_data->identifiers = NULL;;
+	cd_data->identifiers = NULL;
 
 	union node_data nd;
 	nd.cd_data = cd_data;
