@@ -19,6 +19,7 @@
 
 #include "selint_error.h"
 #include "file_list.h"
+#include "string_list.h"
 
 enum selint_error load_access_vectors_kernel(const char *av_path);
 
@@ -32,6 +33,8 @@ enum selint_error load_obj_perm_sets_source(const char *obj_perm_sets_path);
 
 enum selint_error load_devel_headers(struct policy_file_list *context_files);
 
+enum selint_error load_global_conditions(const struct string_list *paths);
+
 enum selint_error mark_transform_interfaces(const struct policy_file_list *files);
 
 #endif
